# -*- coding: utf-8 -*-
import os
import chgender

mcount = 0
fcount = 0
correct = 0


with open(os.getcwd() + '/female.txt', 'r') as f:
    for name in f:
    	name = name.strip('\n')
        fcount = fcount + 1
        if chgender.guess(name)[0] == 'female':
        	correct = correct + 1

with open(os.getcwd() + '/male.txt', 'r') as f:
    for name in f:
    	name = name.strip('\n')
        mcount = mcount + 1
        if chgender.guess(name)[0] == 'male':
        	correct = correct + 1
            
accuracy = 1. * correct/(fcount + mcount)
print('total male input: {} \ntotal female input: {} \naverage accuracy: {}' ''.format(mcount, fcount, accuracy))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

__all__ = ['guess']

class Guesser(object):

    def __init__(self):
    	self._build_lib()
        self._build_model()

    def _build_lib(self):
        with open(os.path.join(os.path.dirname(__file__),'chinese_name_lib.txt'), 'r') as f:  
            data = f.read()  
            self.chinese_name_lib = data.split()

    def _build_model(self):
        self.male_total = 0
        self.female_total = 0
        self.freq = {}

        with open(os.path.join(os.path.dirname(__file__),'pinyin_dataset.txt'), 'r') as f:
            for line in f:
            	line = line.strip('\n')
                pinyin, male, female = line.split()
                self.male_total += int(male)
                self.female_total += int(female)
                self.freq[pinyin] = (int(female), int(male))

        self.total = self.male_total + self.female_total

        for pinyin in self.freq:
            female, male = self.freq[pinyin]
            self.freq[pinyin] = (1. * female / self.female_total,
                               1. * male / self.male_total)

    def guess(self, name):
        name = name.lower()
        full_name  = name.split()
        first_name = full_name[0]
        first_word = ""
        second_word = ""
        i = len(first_name) + 1

        while(i >= 0):
            if first_name[:i] in self.chinese_name_lib:
                if i == len(first_name):
                    first_word = first_name[:i]
                    break
                elif first_name[i:] in self.chinese_name_lib:
				    first_word = first_name[:i]
				    second_word = first_name[i:]
				    break
            i = i - 1

        if first_word == second_word:
        	return ('female', 0.9)
        pf = self.prob_for_gender(first_word, 0) * (self.prob_for_gender(second_word, 0)**2.5)
        pm = self.prob_for_gender(first_word, 1) * (self.prob_for_gender(second_word, 1)**2.5)
        
        if pm > pf:
            return ('male', 1. * pm / (pm + pf))
        elif pm < pf:
            return ('female', 1. * pf / (pm + pf))
        else:
            return ('unknown', 0)

    def prob_for_gender(self, word, gender=0):
    	if word  == "":
    		return 1
        p = 1. * self.female_total / self.total \
            if gender == 0 \
            else 1. * self.male_total / self.total

        p *= self.freq.get(word, (0, 0))[gender]

        return p


guesser = Guesser()
def guess(name):
    return guesser.guess(name)



import argparse
from chgender import guess
import os


def main():
    parser = argparse.ArgumentParser(
        description='Gender guesser for Chinese names in English(pinyin) form')

    group = parser.add_mutually_exclusive_group()
    group.add_argument('-n', '--fnames', nargs='+', help='read first names from arguments')
    group.add_argument('-f',  '--file', nargs=1, help='read first or full names from file(abs dir)')

    parser.add_argument('-v', '--version', action='version', version='chgender_0.0..1')

    args = parser.parse_args()

    if args.fnames:
        for name in args.fnames:
            gender, prob = guess(name)
            print('name: {} => gender: {}, probability: {}'
              ''.format(name, gender, prob))
    elif args.file:
        with open(args.file[0], 'r') as f:
            for name in f:
                name = name.strip('\n')
                gender, prob = guess(name)
                print('name: {} => gender: {}, probability: {}'
                ''.format(name, gender, prob))
    else:
        print('see help: chg -h')


if __name__ == '__main__':
    main()



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='chgender',
    version='0.0.2',
    description='Gender gusser for Chinese names in English (pinyin) form',
    long_description=open('README.md').read(),
    keywords='gender name guess python',
    url='https://github.com/jiajianzhou/chgender',
    author='Jiajian Zhou',
    author_email='jiajianzhou0808@gmail.com',
    packages=['chgender'],
    package_data={'':['*.md','*.txt'],
    'chgender': ['chinese_name_lib.txt','pinyin_dataset.txt']},
    license='MIT License',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python'
    ],
    entry_points={
        'console_scripts': [
            'chg = chgender.batch:main',
        ],
    }
)
